using System.Security.Claims;
using FitConnect.Encryption;
using FitConnect.Models.v1.Api;
using IdentityModel;
using Microsoft.IdentityModel.JsonWebTokens;
using Microsoft.IdentityModel.Tokens;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace FitConnect.Models;

public class SecurityEventToken {
    public const string CreateSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/create-submission";

    public const string SubmitSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/submit-submission";

    public const string NotifySubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/notify-submission";

    public const string ForwardSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/forward-submission";

    public const string RejectSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/reject-submission";

    public const string AcceptSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/accept-submission";

    public const string DeleteSubmissionSchema =
        "https://schema.fitko.de/fit-connect/events/delete-submission";

    private readonly string? _submissionId;

    internal SecurityEventToken() {
    }

    [JsonProperty("iat")]
    public long Iat {
        get => EventTime.ToEpochTime();
        set => EventTime = EpochTime.DateTime(value);
    }

    public DateTime EventTime { get; set; }
    public EventType EventType { get; set; }
    public string EventTypeName => EventType.ToString();
    public Events? Events { get; set; }

    [JsonProperty("sub")]
    private string? SubmissionIdString {
        get => _submissionId?.Contains("submission:") ?? false
            ? _submissionId["submission:".Length..]
            : _submissionId;
        init => _submissionId = $"{value}";
    }

    public Guid SubmissionId =>
        Guid.Parse(SubmissionIdString ?? throw new ArgumentNullException(SubmissionIdString));

    public static async Task<SecurityEventToken> FromJson(string json) {
        //var schema = await JsonSchema.FromUrlAsync(JsonSchemaSettings.SetSchema);
        var schema = await JsonHelper.LoadJsonSchemaFromResource("set-payload.schema.json");
        var validationErrors = schema.Validate(json);
        if (validationErrors.Count > 0) {
            throw new ArgumentException("Not a valid SET json, " +
                                        validationErrors.Aggregate("", (a, b) => a + ", " + b));
        }

        var result = JsonConvert.DeserializeObject<SecurityEventToken>(json) ??
                     throw new ArgumentException("Not a valid SET json");

        result.EventType = EventType.NotSet;
        if (result.Events?.SubmitSubmissionEvent != null) result.EventType = EventType.Submit;
        if (result.Events?.RejectSubmissionEvent != null) result.EventType = EventType.Reject;
        if (result.Events?.CreatedSubmissionEvent != null) result.EventType = EventType.Create;
        if (result.Events?.NotifySubmissionEvent != null) result.EventType = EventType.Notify;
        if (result.Events?.AcceptSubmissionEvent != null) result.EventType = EventType.Accept;
        if (result.Events?.DeleteSubmissionEvent != null) result.EventType = EventType.Delete;

        return result;
    }


    public static async Task<SecurityEventToken> FromJwtEncodedString(string jwt) {
        var token = new JsonWebToken(jwt);
        var json = Base64UrlEncoder.Decode(token.EncodedPayload);
        var result = await FromJson(json);
        return result;
    }

    private EventType DecodeEventType(IEnumerable<Claim> claims) {
        var eventsClaim = claims.FirstOrDefault(c => c.Type == "events");
        if (eventsClaim == null) return EventType.NotSet;

        var events = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(eventsClaim.Value);
        var Payload = JsonConvert
            .DeserializeObject<dynamic>(
                events?.Values?.FirstOrDefault()?.ToString());

        var contentData = JsonConvert.DeserializeObject(eventsClaim.Value);

        if (eventsClaim.Value.Contains(
                CreateSubmissionSchema))
            return EventType.Create;

        if (eventsClaim.Value.Contains(
                SubmitSubmissionSchema))
            return EventType.Submit;

        if (eventsClaim.Value.Contains(
                NotifySubmissionSchema))
            return EventType.Notify;
        if (eventsClaim.Value.Contains(
                ForwardSubmissionSchema))
            return EventType.Forward;
        if (eventsClaim.Value.Contains(
                RejectSubmissionSchema))
            //Problems = GetProblems(events?.Values?.FirstOrDefault()?.ToString() ?? "");
            return EventType.Reject;


        if (eventsClaim.Value.Contains(AcceptSubmissionSchema))

            return EventType.Accept;


        if (eventsClaim.Value.Contains(
                DeleteSubmissionSchema))
            return EventType.Delete;

        return EventType.NotSet;
    }

    private static List<Problems> GetProblems(string payload) {
        if (payload == "") return new List<Problems>();

        var result = (JsonConvert.DeserializeObject<dynamic>(payload)?.problems as JArray)
            ?.Select(j => j as dynamic).ToList()
            ?.Select(p => new Problems {
                Title = p.title,
                Type = p.type,
                Detail = p.detail,
                Instance = p.instance
            })
            ?.ToList() ?? new List<Problems>();
        return result;
    }
}
